using System;
using System.Linq;
using SimpleJSON;

namespace Vinput
{
    public class AutoGensAligner : MVRScript
    {
        // ============================================================================================== //
        // ========================================= PROPERTIES ========================================= //
        // ============================================================================================== //
        public bool __DEBUG__ = false;

        // UI variables
        private JSONStorableBool _activeJSON;

        // Person atoms, controls, colliders
        private TargetManager _targetManager;
        private GensAligner _aligner;
        private FreeControllerV3 _penisBaseCtrl;

        // Temporary controller save/restore settings
        private bool _restoring = false;

        // Getters
        private bool _isValid => _targetManager.TargetRB1 != null && _targetManager.TargetRB2 != null && _penisBaseCtrl != null;


        // ============================================================================================== //
        // ========================================== METHODS =========================================== //
        // ============================================================================================== //

        // ========================================== BUILT-IN ========================================== //
        // Called by game engine on initialization
        public override void Init()
        {
            // Create cycler
            _targetManager = new TargetManager()
            {
                OnTargetChange = _ => { _UpdateState(); }
            };
            _aligner = new GensAligner(containingAtom, _targetManager);


            try
            {
                SetupUI();
            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(AutoGensAligner)}.{nameof(Init)}: {e}");
            }

        }

        // Called by game engine before first update
        private void Start()
        {
            try
            {
                _penisBaseCtrl = containingAtom.freeControllers.First(fc => fc.name == "penisBaseControl");
            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(AutoGensAligner)}: Plugin cannot be applied to an Atom without penisBaseControl.");
                return;
            }

            _UpdateState();

        }

        public void OnEnable() { _UpdateState(); }
        public void OnDisable() { _Deactivate(); }
        public void OnDestroy() { _Deactivate(); }


        // ========================================== CUSTOM ========================================== //


        private void Update() { _aligner.Update(); }

        private void _UpdateState()
        {
            if (!_restoring)
            {
                if (_activeJSON.val && enabled)
                {
                    if (!_aligner.IsEnabled) _aligner.Enable();
                }
                else _Deactivate();
            }
        }



        private void _Deactivate()
        {
            if (_aligner.IsEnabled) _aligner.Disable();
        }

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            var json = base.GetJSON(includePhysical, includeAppearance, forceStore);

            var alignerJson = _aligner.GetJSON();
            if (alignerJson != null) json["AlignerData"] = alignerJson;

            needsStore = true;
            return json;
        }

        public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true)
        {
            _restoring = true;
            base.RestoreFromJSON(jc, restorePhysical, restoreAppearance, presetAtoms, setMissingToDefault);

            if (jc.HasKey("AlignerData")) _aligner.RestoreFromJSON(jc["AlignerData"]);

            _restoring = false;
        }

        private void OnActiveChange(bool _) { _UpdateState(); }
        private void OnAlignPenisChange(bool _) { _UpdateState(); }

        // ========================================== UI ========================================== //
        private void SetupUI()
        {

            _activeJSON = UI_Utils.SetupToggle(this, "<color=#1010A0><size=32><b>Active</b></size></color>", false);
            _activeJSON.setCallbackFunction += OnActiveChange;

            // Target
            _targetManager.SetupUI(this);

            // Instructions
            string instructionText =
                "<color=#202020><size=36><b>Settings Information</b></size></color>\n\n" +
                "<color=#101060><b>Target Settings</b></color>\n" +
                "<b>Target Atom</b>: Name of target Atom.\n" +
                "<b>Target</b>: Anus, Mouth, or Vagina.\n\n" +
                "<color=#101060><b>Adjustments</b></color>\n" +
                "<b>Target Adj. Left-Right</b>: Offsets the target locations left or right, relative to the target.\n" +
                "<b>Target Adj. Down-Up</b>: Offsets the target locations down or up, relative to the target.\n\n";

            CreateTextField(new JSONStorableString("text", instructionText), true).height = 1200;

        }


    }


}
